// Copyright (c) ABBYY (BIT Software), 1993 - 2012. All rights reserved.
// Author: Victor Medvedev
// Target Platfrom: POSIX
// Description: Функции, которые определяют полное имя и хэндл модуля(SharedObject),
// в который они влинкованы.
// Код вынесен в .inl, так как используется более, чем в одном месте,
// но должен быть внутренней линковки - static.
// Используется:
//		PortLayer - ResourcesInit.inl - для инициализации PL ресурсов
//		FineObjects - FineObjStaticUnix.cpp - для регистрации FO и PL модулей
//		PortLayer - Linux/Src/fsTools.cpp - для ответа на запрос CSIDL_COMMON_APPDATA

#pragma once

//--------------------------------------------------------------------------------------------------
// Доступные функции (другие функции напрямую лучше не вызывать):

// Полное имя модуля, так как его видит динамический загрузчик
static const char* getModuleFilename();
// Хэндл модуля, адрес, который возвращает dlopen (вызывается в ::GetModuleHandle())
static HINSTANCE getModuleHandle();

//--------------------------------------------------------------------------------------------------
// Функции определения имени модуля (полное имя файла) и его дескриптора

// Определения имени модуля, в который эта функция влинкована
static NOINLINE_FUNCTION const char* determineModuleFilename()
{
	Dl_info info;
	assert( ::dladdr( (void*)&determineModuleFilename, &info ) != 0 );
	assert( info.dli_fname != 0 );
	static char fullPath[PATH_MAX];
#ifdef _ANDROID
	// Для андроида relpath не может вернуть полный путь, однако это не должно нам помешать в последствии нормально
	// загрузить и идентифицировать библиотеку.
	int i = 0;
	do {
		fullPath[i] = info.dli_fname[i];
		i++;
	} while ( info.dli_fname[i] != '\0' );
#else
#if defined( _OCRT_CONFIG_WITH_LOGGING ) || defined( _MSDK_CONFIG_WITH_LOGGING )
	printf( "determineModuleFilename. Module name: %s.\n", info.dli_fname );
#endif
	assert( ::realpath( info.dli_fname, fullPath ) == fullPath );
#if defined( _OCRT_CONFIG_WITH_LOGGING ) || defined( _MSDK_CONFIG_WITH_LOGGING )
	printf( "determineModuleFilename. Full path: %s.\n", fullPath );
#endif
#endif
	return fullPath;
}

// Определения хэндла модуля. Использует determineModuleFilename() через вызов кэширующей getModuleFilename()
static HINSTANCE determineModuleHandle()
{
	// Определяем имя модуля
	const char* moduleName = getModuleFilename();
	// Запрашиваем хэндл модуля по имени
	HINSTANCE handle = ::GetModuleHandleA( moduleName );
	// ::GetModuleHandle ищет только загруженных модулях на данный момент 
	// Предполагаем, что мы не можем находиться в уже отгруженном модуле
	// Поэтому, если хэндл = 0 (не нашли такого модуля), то считаем, что находимся в выполняемом файле
	if( handle == 0 ) {
		// Определение хэндла исполняемого файла
		handle = ::GetModuleHandleA( 0 );
	}
	return handle;
}

//--------------------------------------------------------------------------------------------------
// Переменные и функции для хранения и получения (кэширования) значений имени модуля и его дескриптора

// Геттер для получения полного имени файла данного модуля
static const char* getModuleFilename()
{
	static const char* moduleFilename = determineModuleFilename();
	assert( moduleFilename != 0 );
#if defined( _OCRT_CONFIG_WITH_LOGGING ) || defined( _MSDK_CONFIG_WITH_LOGGING )
	printf( "getModuleFilename(). moduleFilename: %s.\n", moduleFilename );
#endif
	return moduleFilename;
}

// Геттер для получения хэндла данного модуля
static HINSTANCE getModuleHandle()
{
	static HINSTANCE moduleHandle = determineModuleHandle();
	assert( moduleHandle != 0 );
	return moduleHandle;
}
