// Copyright (c) ABBYY (BIT Software), 1993 - 2012. All rights reserved.
// Автор: Виктор Медведев
// Описание: Инициализация/Деинициализация модулей, регистрации в списке PL и запуск DllMain.
// Идиологически, это часть PortLayer, но чтобы не создавать отдельную подсистему PortLayerStaticPart
// включается в FineObjStaticUnix.cpp и собирается как FineObj в подсистему FineObjStaticPart.
// Отсюда вытекает использование FObj пространства имен

#pragma once

// Warning! Code including.
#include <ModuleInfo.inl>
#include <ResourcesAPI.h> //PortLayer

// Объявление функции DllMain, которая автоматически вызывается при загрузке/отгрузке библиотеки
typedef BOOL WINAPI ( *DllMainFunc )( HINSTANCE hInstance, DWORD dwReason, LPVOID reserved );

// Указатель на функцию DllMain, если существует
static DllMainFunc dllMain = 0;

DLL_CONSTRUCTOR static void OnModuleLoad()
{
	const HINSTANCE moduleHandle = getModuleHandle();
	const FObj::CUnicodeString moduleName( getModuleFilename(), CP_UTF8 );
	DebugTrace( L"OnModuleLoad() [%p, %ls]", moduleHandle, moduleName.Ptr() );

	bool dllMainError = false;
	// Если мы в библиотеке
	if( moduleHandle != ::GetModuleHandleA( 0 ) ) {
		// Ищем и пытаемся запустить DllMain( DLL_PROCESS_ATTACH )
		dllMain = (DllMainFunc) ::GetProcAddress( moduleHandle, "DllMain" );
		
		if( dllMain != 0 ) {
			DebugTrace( L"Run DllMain[%p]( DLL_PROCESS_ATTACH ) from [%p, %ls]", dllMain, moduleHandle, moduleName.Ptr() );
			if( !dllMain( moduleHandle, DLL_PROCESS_ATTACH, 0 ) ) {
				DebugTrace( L"DllMain[%p]( DLL_PROCESS_ATTACH ) from [%p, %ls] returns false", dllMain, moduleHandle, moduleName.Ptr() );
				dllMainError = true;
			}
		}
	}
	// Добавление модуля в список модулей PortLayer
	PortLayer::AddModuleW( moduleHandle, moduleName.Ptr(), dllMainError );
}

// Максимальный приоритет для данной функции деструктора означает, что среди всех других функций
// деструктора в данном DSO/EXE, данная функция будет вызвана последней.
//
// Также, если в данном DSO/EXE имеются объекты со 'static storage duration', объявленные с
// использованием атрибута init_priority, данная функция деструктора будет вызвана после уничтожения
// этих объектов.
//
// Из GCC-документации 
// (https://gcc.gnu.org/onlinedocs/gcc/Common-Function-Attributes.html#Common-Function-Attributes) 
// не совсем понятно специфицируется ли как-то порядок в случае смешанных объявлений, когда
// часть функций деструктора объявляются с указаним приоритета, а часть без, или когда часть объектов
// со 'static storage duration' объявляются с использованием атрибута init_priority, а часть без.
// Локальные эксперименты (Linux_x86_64, g++ 7.5.0) показывают, что объявления с указанием приоритета 
// (кроме самого низкого приоритета 65535) более приоритетны, чем объявления без указания оного.
// Это в частности позволяет надеется, что данная функция деструктора будет вызвана после
// уничтожения объектов со 'static storage duration', независимо от того, объявлены ли они с
// использованием атрибута init_priority или нет.
//
// Один из примеров, когда критично, что данная функция будет вызвана после уничтожения объектов
// со 'static storage duration' подробно описана тут: 
// https://tfs.abbyy.com/HQ/Compact14/_workitems/edit/63843
//
DLL_DESTRUCTOR_PRIO(ATTR_PRIORITY_HIGHEST) static void OnModuleUnload()
{
	HINSTANCE moduleHandle = getModuleHandle();
    // Disable creation of debug message because this function is called when string allocator is already destructed.
    // const FObj::CUnicodeString moduleName( getModuleFilename(), CP_UTF8 );
    // DebugTrace( L"OnModuleUnload() [%p, %ls]", moduleHandle, moduleName.Ptr() );

	// If DllMainFunc defined run it with DLL_PROCESS_DETACH
	if( dllMain != 0 ) {
		// DebugTrace( L"Run DllMain( DLL_PROCESS_DETACH ) from [%p, %ls]", moduleHandle, moduleName.Ptr() );
		if( !dllMain( moduleHandle, DLL_PROCESS_DETACH, 0 ) ) {
			// DebugTrace( L"DllMain( DLL_PROCESS_DETACH ) from [%p, %ls] returns false", moduleHandle, moduleName.Ptr() );
		}
	}
	// Удаление модуля в списка модулей PortLayer
	PortLayer::DeleteModule( moduleHandle );
}
